package cat.hack3.codingtests.marsrover;

import java.util.logging.Logger;

/**
 * Notice that the representation is the following:
 * Latitude is Y, starts from 1 on the top and goes down incrementally
 * Longitude is X, starts from 1 on the left and goes to right incrementally
 *     1    2    3
 * 1  1-1  1-2  1-3
 * 2  2-1  2-2  2-3
 * 3  3-1  3-2  3-3
 */
public class MarsMap {
    Logger logger = Logger.getLogger(this.getClass().getName());

    private final int width;
    private final int height;
    private Coordinates currentPosition;

    public MarsMap(int width, int height, Coordinates startingCoordinates) {
        this.width = width;
        this.height = height;
        currentPosition = startingCoordinates;
    }

    public Coordinates updatePositionTowards(Direction direction) {
        switch (direction) {
            case NORTH -> decrementLatitude();
            case SOUTH -> incrementLatitude();
            case WEST -> decrementLongitude();
            case EAST -> incrementLongitude();
        }
        logger.info(String.format("Updated coordinates towards %s, now is %d-%d",
                direction, currentPosition.getLatitude(), currentPosition.getLongitude()));

        return currentPosition;
    }

    private void incrementLatitude() {
        int latitude = currentPosition.getLatitude();
        int newLatitude = latitude + 1 > height
                ? 1
                : ++latitude;
        setNewLatitude(newLatitude);
    }

    private void decrementLatitude() {
        int latitude = currentPosition.getLatitude();
        int newLatitude = latitude - 1 < 1
                ? height
                : --latitude;
        setNewLatitude(newLatitude);
    }

    private void setNewLatitude(int newLatitude) {
        currentPosition = currentPosition.ofUpdatedLatitude(newLatitude);
    }

    private void incrementLongitude() {
        int longitude = currentPosition.getLongitude();
        int newLongitude = longitude + 1 > width
                ? 1
                : ++longitude;
        setNewLongitude(newLongitude);
    }

    private void setNewLongitude(int newLongitude) {
        currentPosition = currentPosition.ofUpdatedLongitude(newLongitude);
    }

    private void decrementLongitude() {
    }

    public Coordinates getCurrentPosition() {
        return currentPosition;
    }
}
